{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Colas\n",
    "\n",
    "## Objetivos\n",
    "\n",
    "  - Comprender el concepto y las características de la estructura de datos **Cola**.\n",
    "  - Implementar colas en Java utilizando **arreglos** y **nodos (listas enlazadas)**.\n",
    "  - Practicar el uso de operaciones básicas como **enqueue** y **dequeue**.\n",
    "  - Aplicar la estructura de colas en ejemplos prácticos.\n",
    "\n",
    "## Introducción\n",
    "\n",
    "En la programación y ciencias de la computación, una **cola** (Queue) es una estructura de datos lineal que sigue el principio **FIFO** (First In, First Out). Esto significa que el primer elemento en entrar es el primero en salir, tal como una fila en una tienda o en un banco. La cola tiene dos operaciones principales:\n",
    "\n",
    "  - **Encolar (enqueue)**: añadir un elemento al final de la cola.\n",
    "  - **Desencolar (dequeue)**: eliminar el elemento al frente de la cola.\n",
    "\n",
    "Las colas son muy utilizadas en problemas relacionados con sistemas de gestión de tareas, simulaciones y procesamiento de datos, entre otros.\n",
    "\n",
    "## Concepto de Cola\n",
    "\n",
    "Una cola es una colección ordenada de elementos donde el acceso es restringido. Se caracteriza por:\n",
    "\n",
    "  1. **Inserción de elementos** al final (también llamado \"cola\" o \"final\").\n",
    "  2. **Eliminación de elementos** desde el frente (el primero en entrar).\n",
    "\n",
    "El comportamiento FIFO es ideal en situaciones donde se requiere un procesamiento por orden de llegada. Las colas tienen una amplia variedad de implementaciones, incluyendo **colas circulares**, **colas de prioridad**, y **doble cola** (deque), pero aquí nos enfocaremos en la cola básica."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "\n",
    "## Implementaciones de Colas en Java\n",
    "\n",
    "### Usando Arreglos\n",
    "\n",
    "La implementación de colas con **arreglos** es directa, pero tiene una desventaja: el tamaño de la cola es fijo y si no se utiliza adecuadamente puede haber **desperdicio de espacio**. A continuación, se muestra cómo se puede implementar una cola usando arreglos:\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class ColaArray {\n",
    "  private int[] cola;\n",
    "  private int frente;\n",
    "  private int finalCola;\n",
    "  private int capacidad;\n",
    "  \n",
    "  // Constructor para inicializar la cola\n",
    "  public ColaArray(int tamano) {\n",
    "      capacidad = tamano;\n",
    "      cola = new int[capacidad];\n",
    "      frente = 0;\n",
    "      finalCola = -1;\n",
    "  }\n",
    "  \n",
    "  // Método para encolar (agregar) un elemento\n",
    "  public void enqueue(int elemento) {\n",
    "      if (finalCola == capacidad - 1) {\n",
    "          System.out.println(\"Cola llena\");\n",
    "          return;\n",
    "      }\n",
    "      cola[++finalCola] = elemento;\n",
    "  }\n",
    "  \n",
    "  // Método para desencolar (eliminar) un elemento\n",
    "  public int dequeue() {\n",
    "      if (estaVacia()) {\n",
    "          System.out.println(\"Cola vacía\");\n",
    "          return -1;\n",
    "      }\n",
    "      return cola[frente++];\n",
    "  }\n",
    "  \n",
    "  // Verificar si la cola está vacía\n",
    "  public boolean estaVacia() {\n",
    "      return frente > finalCola;\n",
    "  }\n",
    "\n",
    "  // Método para obtener el frente de la cola sin desencolarlo\n",
    "  public int frente() {\n",
    "      if (estaVacia()) {\n",
    "          System.out.println(\"Cola vacía\");\n",
    "          return -1;\n",
    "      }\n",
    "      return cola[frente];\n",
    "  }\n",
    "}\n",
    "\n",
    "public class Main {\n",
    "  public static void main(String[] args) {\n",
    "      ColaArray cola = new ColaArray(5);\n",
    "      cola.enqueue(10);\n",
    "      cola.enqueue(20);\n",
    "      cola.enqueue(30);\n",
    "      \n",
    "      System.out.println(\"Desencolado: \" + cola.dequeue());\n",
    "      System.out.println(\"Frente actual: \" + cola.frente());\n",
    "  }\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Modifica el anterior código: agrega más elementos, elimina, imprime, etc. Además, crea objetos y prueba la clase"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "// Prueba tu código aquí"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Explicación del código:\n",
    "\n",
    "- **enqueue()**: Añade un nuevo elemento al final de la cola. Si la cola está llena, se muestra un mensaje.\n",
    "- **dequeue()**: Elimina y devuelve el elemento al frente de la cola. Si la cola está vacía, devuelve -1.\n",
    "- **frente()**: Devuelve el elemento del frente sin eliminarlo.\n",
    "- **estaVacia()**: Comprueba si la cola está vacía.\n",
    "\n",
    "**Desventaja:**\n",
    "- En este caso, una vez que se ha encolado y desencolado un número determinado de elementos, la cola deja de funcionar correctamente porque los índices `frente` y `finalCola` siguen incrementándose sin reusarse. Esto puede solucionarse con una **cola circular**.\n",
    "\n",
    "\n",
    "### Usando Nodos (Listas Enlazadas)\n",
    "\n",
    "Otra forma eficiente de implementar una cola es mediante una **lista enlazada** donde los nodos almacenan datos y una referencia al siguiente nodo en la cola. Este enfoque no tiene un tamaño fijo como los arreglos y se adapta mejor al uso dinámico de la memoria."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class Nodo {\n",
    "  int dato;\n",
    "  Nodo siguiente;\n",
    "\n",
    "  // Constructor del nodo\n",
    "  public Nodo(int dato) {\n",
    "      this.dato = dato;\n",
    "      this.siguiente = null;\n",
    "  }\n",
    "}\n",
    "\n",
    "class ColaListaEnlazada {\n",
    "  private Nodo frente, finalCola;\n",
    "\n",
    "  // Constructor para inicializar la cola\n",
    "  public ColaListaEnlazada() {\n",
    "      this.frente = null;\n",
    "      this.finalCola = null;\n",
    "  }\n",
    "\n",
    "  // Método para encolar un elemento\n",
    "  public void enqueue(int dato) {\n",
    "      Nodo nuevoNodo = new Nodo(dato);\n",
    "      if (finalCola == null) {\n",
    "          frente = finalCola = nuevoNodo;\n",
    "      } else {\n",
    "          finalCola.siguiente = nuevoNodo;\n",
    "          finalCola = nuevoNodo;\n",
    "      }\n",
    "  }\n",
    "\n",
    "  // Método para desencolar un elemento\n",
    "  public int dequeue() {\n",
    "      if (frente == null) {\n",
    "          System.out.println(\"Cola vacía\");\n",
    "          return -1;\n",
    "      }\n",
    "      int dato = frente.dato;\n",
    "      frente = frente.siguiente;\n",
    "      if (frente == null) {\n",
    "          finalCola = null;\n",
    "      }\n",
    "      return dato;\n",
    "  }\n",
    "\n",
    "  // Verificar si la cola está vacía\n",
    "  public boolean estaVacia() {\n",
    "      return frente == null;\n",
    "  }\n",
    "\n",
    "  // Método para obtener el frente de la cola sin desencolarlo\n",
    "  public int frente() {\n",
    "      if (frente == null) {\n",
    "          System.out.println(\"Cola vacía\");\n",
    "          return -1;\n",
    "      }\n",
    "      return frente.dato;\n",
    "  }\n",
    "}\n",
    "\n",
    "public class Main {\n",
    "  public static void main(String[] args) {\n",
    "      ColaListaEnlazada cola = new ColaListaEnlazada();\n",
    "      cola.enqueue(10);\n",
    "      cola.enqueue(20);\n",
    "      cola.enqueue(30);\n",
    "      \n",
    "      System.out.println(\"Frente actual: \" + cola.frente());\n",
    "      System.out.println(\"Desencolado: \" + cola.dequeue());\n",
    "      System.out.println(\"Frente actual después de desencolar: \" + cola.frente());\n",
    "  }\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Explicación del código:\n",
    "- **Nodo**: Cada nodo de la cola contiene el dato y una referencia al siguiente nodo.\n",
    "- **enqueue()**: Inserta un nuevo nodo al final de la cola.\n",
    "- **dequeue()**: Elimina y devuelve el nodo al frente.\n",
    "- **estaVacia()**: Comprueba si la cola está vacía.\n",
    "\n",
    "**Ventaja:**\n",
    "- La lista enlazada permite crear colas dinámicas de tamaño ilimitado, ya que no depende de un tamaño de arreglo fijo."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "// Prueba tu código aquí"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Usando la interfaz de Java\n",
    "\n",
    "\n",
    "#### Colas\n",
    "\n",
    "Utilizando la clase de java [Queue.java](https://github.com/openjdk/jdk/blob/master/src/java.base/share/classes/java/util/Queue.java), implementen una cola con el tema que más les guste.\n",
    "\n",
    "```{figure} ../../images/Figure20.10.png\n",
    "---\n",
    "width: 90%\n",
    "name: Figure20.12\n",
    "---\n",
    "Here is my figure caption!\n",
    "```\n",
    "\n",
    "```{figure} ../../images/Figure20.13.png\n",
    "---\n",
    "width: 90%\n",
    "name: Figure20.13\n",
    "---\n",
    "Here is my figure caption!\n",
    "```\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import java.util.Collection;\n",
    "import java.util.Iterator;\n",
    "import java.util.LinkedList;\n",
    "import java.util.NoSuchElementException;\n",
    "import java.util.Queue;\n",
    "\n",
    "public class Fila<T> implements Queue<T> {\n",
    "\n",
    "    private LinkedList<T> elementos;\n",
    "\n",
    "    public Fila() {\n",
    "        elementos = new LinkedList<>();\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean add(T e) {\n",
    "        return elementos.add(e);  // Agrega un elemento al final de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean offer(T e) {\n",
    "        return elementos.add(e);  // Similar a add, pero no lanza una excepción si falla\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public T remove() {\n",
    "        if (elementos.isEmpty()) {\n",
    "            throw new NoSuchElementException(\"La cola está vacía\");\n",
    "        }\n",
    "        return elementos.removeFirst();  // Remueve el primer elemento de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public T poll() {\n",
    "        return elementos.isEmpty() ? null : elementos.removeFirst();  // Devuelve y remueve el primer elemento o null si está vacía\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public T element() {\n",
    "        if (elementos.isEmpty()) {\n",
    "            throw new NoSuchElementException(\"La cola está vacía\");\n",
    "        }\n",
    "        return elementos.getFirst();  // Devuelve el primer elemento sin removerlo\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public T peek() {\n",
    "        return elementos.isEmpty() ? null : elementos.getFirst();  // Devuelve el primer elemento o null si está vacía\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public int size() {\n",
    "        return elementos.size();  // Devuelve el tamaño de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean isEmpty() {\n",
    "        return elementos.isEmpty();  // Verifica si la cola está vacía\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean contains(Object o) {\n",
    "        return elementos.contains(o);  // Verifica si la cola contiene el elemento\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public Iterator<T> iterator() {\n",
    "        return elementos.iterator();  // Devuelve un iterador para recorrer los elementos de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public Object[] toArray() {\n",
    "        return elementos.toArray();  // Convierte la cola a un array de objetos\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public <U> U[] toArray(U[] a) {\n",
    "        return elementos.toArray(a);  // Convierte la cola a un array de un tipo específico\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean remove(Object o) {\n",
    "        return elementos.remove(o);  // Remueve un objeto de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean containsAll(Collection<?> c) {\n",
    "        return elementos.containsAll(c);  // Verifica si la cola contiene todos los elementos de la colección dada\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean addAll(Collection<? extends T> c) {\n",
    "        return elementos.addAll(c);  // Agrega todos los elementos de la colección dada a la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean removeAll(Collection<?> c) {\n",
    "        return elementos.removeAll(c);  // Remueve todos los elementos de la colección dada de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public boolean retainAll(Collection<?> c) {\n",
    "        return elementos.retainAll(c);  // Retiene solo los elementos presentes en la colección dada\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public void clear() {\n",
    "        elementos.clear();  // Limpia todos los elementos de la cola\n",
    "    }\n",
    "\n",
    "    @Override\n",
    "    public String toString() {\n",
    "        return elementos.toString();  // Convierte la lista de elementos a una cadena y la devuelve\n",
    "    }\n",
    "}"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[Sebas, tian, Aguilera]\n",
      "Primer elemento: Sebas\n",
      "Elemento removido: Sebas\n",
      "¿Contiene el número 2? false\n",
      "Tamaño de la cola: 2\n",
      "[tian, Aguilera]\n"
     ]
    }
   ],
   "source": [
    "public class Main {\n",
    "  public static void main(String[] args) {\n",
    "      Fila<String> fila = new Fila<>();\n",
    "\n",
    "      // Agregar elementos a la cola\n",
    "      fila.add(\"Sebas\");\n",
    "      fila.add(\"tian\");\n",
    "      fila.add(\"Aguilera\");\n",
    "\n",
    "      System.out.println(fila);\n",
    "      // Ver el primer elemento sin removerlo\n",
    "      System.out.println(\"Primer elemento: \" + fila.peek());\n",
    "\n",
    "      // Remover el primer elemento de la cola\n",
    "      System.out.println(\"Elemento removido: \" + fila.remove());\n",
    "\n",
    "      // Verificar si contiene el número 2\n",
    "      System.out.println(\"¿Contiene el número 2? \" + fila.contains(2));\n",
    "\n",
    "      // Ver el tamaño de la cola\n",
    "      System.out.println(\"Tamaño de la cola: \" + fila.size());\n",
    "\n",
    "      System.out.println(fila);\n",
    "  }\n",
    "}\n",
    "new Main().main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Colas Prioritarias\n",
    "\n",
    "```{figure} https://simplerize.com/wp-content/uploads/ds/queue/priority/priority-queue-diagram.svg\n",
    "---\n",
    "width: 90%\n",
    "name: pilaprioritaria\n",
    "---\n",
    "Ejemplo pila prioritaria de números.\n",
    "```\n",
    "\n",
    "```{figure} https://i0.wp.com/learnersbucket.com/wp-content/uploads/2019/09/ezgif.com-optimize-2.gif?resize=600%2C338&ssl=1\n",
    "---\n",
    "width: 90%\n",
    "name: Figure20.14\n",
    "---\n",
    "Método de organización de las pilas prioritarias.\n",
    "```\n",
    "\n",
    "\n",
    "\n",
    "Utilizando la clase de java [PriorityQueue.java](https://github.com/openjdk/jdk/blob/master/src/java.base/share/classes/java/util/PriorityQueue.java), implementen una cola con el tema que más les guste.\n",
    "\n",
    "```{figure} ../../images/Figure20.14.png\n",
    "---\n",
    "width: 90%\n",
    "name: Figure20.14\n",
    "---\n",
    "Here is my figure caption!\n",
    "```\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "// Prueba tu código aquí"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Otros Tipos de Colas\n",
    "\n",
    "### Colas Circulares\n",
    "\n",
    "<div align=\"center\">\n",
    "  <img src=\"https://www.atatus.com/glossary/content/images/2022/09/Circular-Queue-1.png\" width=40%>\n",
    "</div>\n",
    "\n",
    "\n",
    "### Cola de Doble Extremo\n",
    "\n",
    "<div align=\"center\">\n",
    "  <img src=\"https://www.atatus.com/glossary/content/images/size/w1000/2022/09/Deque.png\" width=80%>\n",
    "</div>\n",
    "\n",
    "## Aplicación de Colas\n",
    "\n",
    "Las colas son útiles en muchos problemas de la vida real y en la programación. Algunos ejemplos incluyen:\n",
    "\n",
    "1. **Gestión de tareas** en sistemas operativos: Las colas gestionan tareas que se deben procesar de acuerdo al orden de llegada.\n",
    "2. **Sistemas de impresión**: Una impresora puede utilizar una cola para manejar múltiples trabajos de impresión en orden.\n",
    "3. **Sistemas de soporte al cliente**: Para atender solicitudes de los clientes en orden de llegada.\n",
    "4. **Simulaciones**: En simulaciones de procesos industriales, como líneas de ensamblaje o tráfico."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Ejercicio: Aplicación de Pilas y Colas a un Problema de Interés\n",
    "\n",
    "::::{admonition} Taller 6\n",
    "\n",
    "El objetivo de este taller es que los estudiantes apliquen las estructuras de datos **Pilas (Stacks)** y **Colas (Queues)** a un problema real de su interés. Deberán analizar y proponer una solución que emplee ambas estructuras, identificar las ventajas y desventajas de cada una, y decidir cuál es más adecuada para resolver el problema en cuestión.\n",
    "\n",
    "### Instrucciones\n",
    "\n",
    "1. **Escoge un problema**: Piensa en un problema de la vida real o de un ámbito que te apasione, que pueda modelarse con una estructura de datos como una pila o una cola. Algunos ejemplos pueden ser:\n",
    "   \n",
    "   - Gestión de tareas en una aplicación de productividad.\n",
    "   - Organización de tickets en una cola de atención al cliente.\n",
    "   - Control de operaciones matemáticas (como evaluaciones de expresiones).\n",
    "   - Sistemas de llamadas en una central telefónica.\n",
    "   - Gestión de pacientes en una sala de espera de hospital.\n",
    "   - Un videojuego donde se maneja el inventario de objetos del jugador.\n",
    "   \n",
    "   **Sugerencias de Problemas**\n",
    "\n",
    "   Si tienes dificultades para encontrar un problema que te apasione, aquí tienes algunas sugerencias:\n",
    "   1. **Sistema de colas en un banco**: Simula la llegada de clientes a un banco y cómo son atendidos según una cola o una pila.\n",
    "   2. **Sistema de deshacer y rehacer en un editor de texto**: Modela las operaciones de deshacer (ctrl+z) y rehacer (ctrl+y) utilizando una pila.\n",
    "   3. **Entrega de paquetes en una empresa de logística**: Modela el sistema de organización y entrega de paquetes según una cola o una pila.\n",
    "   4. **Simulación de un parqueadero**: Simula cómo entran y salen los autos de un parqueadero usando una pila o una cola.\n",
    "   5. **Juego de cartas**: Modela el manejo de una baraja de cartas, donde el jugador extrae o apila cartas en función de una pila o una cola.\n",
    "   \n",
    "2. **Plantea el problema**: Define claramente cuál es el problema que quieres resolver. Algunos elementos a considerar:\n",
    "   - ¿Qué tipo de datos se gestionan?\n",
    "   - ¿Cómo deben ser organizados los datos?\n",
    "   - ¿Cuál es el orden de procesamiento de los datos (último en entrar, primero en salir; o primero en entrar, primero en salir)?\n",
    "\n",
    "3. **Describe el uso de una pila (Stack)**: Explica cómo resolverías tu problema usando una **Pila**. Describe en detalle:\n",
    "   - ¿Por qué utilizarías una pila?\n",
    "   - ¿Cuál sería la lógica detrás de la solución basada en LIFO (Last In, First Out)?\n",
    "   - ¿Qué tipo de operaciones harías con la pila? (inserción, eliminación, consulta)\n",
    "   - Ventajas y desventajas de este enfoque en tu problema.\n",
    "\n",
    "4. **Describe el uso de una cola (Queue)**: Explica cómo resolverías tu problema utilizando una **Cola**. Describe en detalle:\n",
    "   - ¿Por qué utilizarías una cola?\n",
    "   - ¿Cuál sería la lógica detrás de la solución basada en FIFO (First In, First Out)?\n",
    "   - ¿Qué tipo de operaciones harías con la cola? (inserción, eliminación, consulta)\n",
    "   - Ventajas y desventajas de este enfoque en tu problema.\n",
    "\n",
    "5. **Comparación**: Realiza una comparación entre ambos enfoques (uso de pila vs. uso de cola):\n",
    "   - ¿Cuál estructura de datos es más eficiente o apropiada para resolver el problema que escogiste?\n",
    "   - ¿Existen limitaciones o problemas específicos con cada enfoque?\n",
    "   - ¿Cuál es tu recomendación final sobre cuál estructura de datos es la mejor para este caso y por qué?\n",
    "\n",
    "6. **Conclusiones**: Reflexiona sobre lo que has aprendido en este proceso. Considera las siguientes preguntas:\n",
    "   - ¿Qué te ha aportado aplicar estos conceptos a un problema de tu interés?\n",
    "   - ¿Cómo las diferentes estructuras de datos permiten resolver problemas de manera más o menos eficiente?\n",
    "   - ¿Qué aprendiste sobre el proceso de elegir la estructura de datos adecuada para un problema específico?\n",
    "\n",
    "### Entregables\n",
    "\n",
    "1. **Descripción del problema**: Explicación clara del problema elegido.\n",
    "2. **Solución basada en pila**: Descripción detallada del uso de la pila para resolver el problema, incluyendo sus ventajas y desventajas.\n",
    "3. **Solución basada en cola**: Descripción detallada del uso de la cola para resolver el problema, incluyendo sus ventajas y desventajas.\n",
    "4. **Comparación entre ambos enfoques**: Comparación estructurada entre las dos soluciones.\n",
    "5. **Conclusiones**: Reflexión personal sobre el ejercicio, los aprendizajes obtenidos y el proceso de decisión.\n",
    "\n",
    "\n",
    "### Criterios de Evaluación\n",
    "\n",
    "- **Creatividad y originalidad** del problema propuesto.\n",
    "- **Claridad en la descripción** del problema y en la solución propuesta con pilas y colas.\n",
    "- **Profundidad de la comparación** entre ambas estructuras de datos.\n",
    "- **Reflexión y análisis** en las conclusiones.\n",
    "\n",
    ":::"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Conclusiones\n",
    "\n",
    "La estructura de datos **cola** es una herramienta fundamental en programación, con aplicaciones en numerosos sistemas y algoritmos. Entender su implementación tanto con **arreglos** como con **nodos** (listas enlazadas) es esencial para trabajar eficientemente en proyectos que requieran gestionar el orden de ejecución o procesamiento. Su principio **FIFO** la hace ideal para situaciones en las que el primer elemento en ingresar debe ser el primero en ser procesado.\n",
    "\n",
    "## Recursos Adicionales\n",
    "\n",
    "- [Queue - atatus](https://www.atatus.com/glossary/queue/)\n",
    "- [Queue Interface In Java](https://www.geeksforgeeks.org/queue-interface-java/)\n",
    "- [Java Queue – Queue in Java](https://www.digitalocean.com/community/tutorials/java-queue)\n",
    "- [Java Queue Interface](https://www.javatpoint.com/java-priorityqueue)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Java",
   "language": "java",
   "name": "java"
  },
  "language_info": {
   "codemirror_mode": "java",
   "file_extension": ".jshell",
   "mimetype": "text/x-java-source",
   "name": "java",
   "pygments_lexer": "java",
   "version": "21.0.4+7-Ubuntu-1ubuntu224.04"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
